import datetime
import re
import subprocess
import sys
import time
import threading

ASPEN1 = "ASPEN1"
ASPEN2 = "ASPEN2"
programName = "/opt/iskratel/sbin/summ"
g_buff = []
t_quit_sign = True
p = subprocess.Popen(programName, stdin=subprocess.PIPE, shell=True)
p.stdin.write(b"Base 800000000000\n")
p.stdin.flush()
time.sleep(0.1)

def read_registers(lock):
    global g_buff, t_quit_sign
    pPipe = subprocess.Popen(programName, stdin=subprocess.PIPE, stdout=subprocess.PIPE, shell=True, bufsize=4096, start_new_session=True)
    pPipe.stdin.write(b"Base 800000000000\n")
    pPipe.stdin.flush()
    time.sleep(0.1)
    oldtime = datetime.datetime.now()
    while True:
        out = pPipe.stdout.readline().splitlines()
        if out.pop() == b"(base=0x0) => Base 800000000000":
            break
        newtime = datetime.datetime.now()
        if (newtime - oldtime).seconds > 5:
            pPipe.stdin.close()
            pPipe.terminate()
            t_quit_sign = False
            exit(0)
    pPipe.stdin.write(b"md.b 2d 8\n")
    pPipe.stdin.flush()
    time.sleep(0.1)
    while True:
        oldtime = datetime.datetime.now()
        out = pPipe.stdout.readline().splitlines()
        lock.acquire()
        g_buff.append(out.pop())
        lock.release()
        newtime = datetime.datetime.now()
        if (newtime-oldtime).seconds > 5:
            break
    pPipe.stdin.close()
    pPipe.terminate()


def set_PON_MODE_SEL(ASPEN, hex_num):
    if ASPEN == ASPEN1:
        p.stdin.write(b"mpm.b 2f\n")
    if ASPEN == ASPEN2:
        p.stdin.write(b"mpm.b 30\n")
    p.stdin.flush()
    time.sleep(0.1)
    value = str(hex(int(hex_num, 16))[2:].zfill(2)).encode("utf-8")
    p.stdin.write(b"%b\n" % bytes(value))
    p.stdin.flush()
    time.sleep(0.1)
    p.stdin.write(b".\n")
    p.stdin.flush()
    time.sleep(0.1)

def set_PIN1_SW_IN(ASPEN, hex_num):
    if ASPEN == ASPEN1:
        p.stdin.write(b"mpm.b 31\n")
    if ASPEN == ASPEN2:
        p.stdin.write(b"mpm.b 32\n")
    p.stdin.flush()
    time.sleep(0.1)
    value = str(hex(int(hex_num, 16))[2:].zfill(2)).encode("utf-8")
    p.stdin.write(b"%b\n" % bytes(value))
    p.stdin.flush()
    time.sleep(0.1)
    p.stdin.write(b".\n")
    p.stdin.flush()
    time.sleep(0.1)

def set_PIN9_SW_IN(ASPEN, hex_num):
    if ASPEN == ASPEN1:
        p.stdin.write(b"mpm.b 33\n")
    if ASPEN == ASPEN2:
        p.stdin.write(b"mpm.b 34\n")
    p.stdin.flush()
    time.sleep(0.1)
    value = str(hex(int(hex_num, 16))[2:].zfill(2)).encode("utf-8")
    p.stdin.write(b"%b\n" % bytes(value))
    p.stdin.flush()
    time.sleep(0.1)
    p.stdin.write(b".\n")
    p.stdin.flush()
    time.sleep(0.1)

def set_TX_DIS_CTRLR(ASPEN, hex_num):
    if ASPEN == ASPEN1:
        p.stdin.write(b"mpm.b 2d\n")
    if ASPEN == ASPEN2:
        p.stdin.write(b"mpm.b 2e\n")
    p.stdin.flush()
    time.sleep(0.1)
    value = str(hex(int(hex_num, 16))[2:].zfill(2)).encode("utf-8")
    p.stdin.write(b"%b\n" % bytes(value))
    p.stdin.flush()
    time.sleep(0.1)
    p.stdin.write(b".\n")
    p.stdin.flush()
    time.sleep(0.1)

def set_PON_TYPE_by_OLTMNG(phyLinkId, PonType):
    print("\nSWITCH PONTYPE IN OLTMNG\n")
    virLinkId = phyLinkId * 2 - 1
    pPipe = subprocess.Popen("telnet localhost 2024", stdin=subprocess.PIPE, shell=True)

    pPipe.stdin.write(b'SendOltPonLinkCfgTest(3,1,1,%d,0,0,0,1,0x00004000,%d);\n' % (virLinkId, PonType))
    pPipe.stdin.flush()
    time.sleep(5)
    pPipe.terminate()

def cpld_quit():
    try:
        p.stdin.close()
        p.terminate()
    except Exception as e:
        print("catch some exceptions: %s\n" % e)
    finally:
        p.kill()

def main():
    print("\nUsage: python3 cpld_settings.py PON_TYPE PHY_PORT\n")
    print("\nUsage: python3 cpld_settings.py PON_TYPE PHY_PORT\n")
    if len(sys.argv) < 3:
        print("\nPlease enter correct numbers of parameters.\n")
        cpld_quit()
        exit(0)
    pon_type = int(sys.argv[1])
    phy_port = int(sys.argv[2])
    if pon_type <= 0 or pon_type > 3:
        print("\nPON_TYPE: 1(gpon_xgs) 2(xgs_multirate) 3(combo)\n")
        cpld_quit()
        exit(0)
    if phy_port <= 0 or phy_port > 16:
        print("\nPHY_PORT: 1~16\n")
        cpld_quit()
        exit(0)

    bit_2d = 0xff
    bit_2e = 0xff
    bit_2f = 0xff
    bit_30 = 0xff
    bit_31 = 0xff
    bit_32 = 0xff
    bit_33 = 0xff
    bit_34 = 0xff
    tmp_bit_2d = []
    tmp_bit_2e = []
    tmp_bit_2f = []
    tmp_bit_30 = []
    tmp_bit_31 = []
    tmp_bit_32 = []
    tmp_bit_33 = []
    tmp_bit_34 = []
    choose_aspen = ASPEN1

    set_PON_TYPE_by_OLTMNG(phy_port, pon_type)
    lock = threading.Lock()
    t = threading.Thread(target=read_registers, args=(lock, ))
    t.setDaemon(True)
    t.start()
    t.join(timeout=5)

    if t_quit_sign == False:
        cpld_quit()
        print("ERROR: get cpld settings failed.\n")
        exit(-1)
    g_buff.pop()
    tmp1 = str(g_buff.pop().decode("utf-8"))
    tmp2 = str(g_buff.pop().decode("utf-8"))

    if re.search("800000000030", tmp1):
        bit_30 = hex(int("0x" + tmp1.split(":")[1].split(" ")[1], 16))
        bit_31 = hex(int("0x" + tmp1.split(":")[1].split(" ")[2], 16))
        bit_32 = hex(int("0x" + tmp1.split(":")[1].split(" ")[3], 16))
        bit_33 = hex(int("0x" + tmp1.split(":")[1].split(" ")[4], 16))
        bit_34 = hex(int("0x" + tmp1.split(":")[1].split(" ")[5], 16))

    if re.search("800000000020", tmp2):
        bit_2d = hex(int("0x" + tmp2.split(":")[1].split(" ")[14], 16))
        bit_2e = hex(int("0x" + tmp2.split(":")[1].split(" ")[15], 16))
        bit_2f = hex(int("0x" + tmp2.split(":")[1].split(" ")[16], 16))

    if phy_port > 8:
        choose_aspen = ASPEN2
        phy_port = phy_port - 8
        tmp_bit_2e = list(bin(int(bit_2e, 16))[2:].zfill(8))
        tmp_bit_30 = list(bin(int(bit_30, 16))[2:].zfill(8))
        tmp_bit_32 = list(bin(int(bit_32, 16))[2:].zfill(8))
        tmp_bit_34 = list(bin(int(bit_34, 16))[2:].zfill(8))
        # GPON_XGSPON
        if pon_type == 1:
            tmp_bit_2e[8 - phy_port] = 0
            tmp_bit_30[8 - phy_port] = 1
            tmp_bit_32[8 - phy_port] = 1
            tmp_bit_34[8 - phy_port] = 1
        # XGSPON_multirate
        elif pon_type == 2:
            tmp_bit_2e[8 - phy_port] = 0
            tmp_bit_30[8 - phy_port] = 1
            tmp_bit_32[8 - phy_port] = 0
            tmp_bit_34[8 - phy_port] = 1
        # COMBO
        else:
            tmp_bit_2e[8 - phy_port] = 0
            tmp_bit_30[8 - phy_port] = 0
            tmp_bit_32[8 - phy_port] = 1
            tmp_bit_34[8 - phy_port] = 1
    else:
        choose_aspen = ASPEN1
        tmp_bit_2d = list(bin(int(bit_2d, 16))[2:].zfill(8))
        tmp_bit_2f = list(bin(int(bit_2f, 16))[2:].zfill(8))
        tmp_bit_31 = list(bin(int(bit_31, 16))[2:].zfill(8))
        tmp_bit_33 = list(bin(int(bit_33, 16))[2:].zfill(8))
        # GPON_XGSPON
        if pon_type == 1:
            tmp_bit_2d[8 - phy_port] = 0
            tmp_bit_2f[8 - phy_port] = 1
            tmp_bit_31[8 - phy_port] = 1
            tmp_bit_33[8 - phy_port] = 1
        # XGSPON_multirate
        elif pon_type == 2:
            tmp_bit_2d[8 - phy_port] = 0
            tmp_bit_2f[8 - phy_port] = 1
            tmp_bit_31[8 - phy_port] = 0
            tmp_bit_33[8 - phy_port] = 1
        # COMBO
        else:
            tmp_bit_2d[8 - phy_port] = 0
            tmp_bit_2f[8 - phy_port] = 0
            tmp_bit_31[8 - phy_port] = 1
            tmp_bit_33[8 - phy_port] = 1


    if choose_aspen == ASPEN1:
        bit_2d = hex(int("".join(str(x) for x in tmp_bit_2d), 2))
        bit_2f = hex(int("".join(str(x) for x in tmp_bit_2f), 2))
        bit_31 = hex(int("".join(str(x) for x in tmp_bit_31), 2))
        bit_33 = hex(int("".join(str(x) for x in tmp_bit_33), 2))
        print("ASPEN1:", bit_2d, bit_2f, bit_31, bit_33)
        set_TX_DIS_CTRLR(choose_aspen, bit_2d)
        set_PON_MODE_SEL(choose_aspen, bit_2f)
        set_PIN1_SW_IN(choose_aspen, bit_31)
        set_PIN9_SW_IN(choose_aspen, bit_33)
    elif choose_aspen == ASPEN2:
        bit_2e = hex(int("".join(str(x) for x in tmp_bit_2e), 2))
        bit_30 = hex(int("".join(str(x) for x in tmp_bit_30), 2))
        bit_32 = hex(int("".join(str(x) for x in tmp_bit_32), 2))
        bit_34 = hex(int("".join(str(x) for x in tmp_bit_34), 2))
        print("ASPEN2:", bit_2e, bit_30, bit_32, bit_34)
        set_TX_DIS_CTRLR(choose_aspen, bit_2e)
        set_PON_MODE_SEL(choose_aspen, bit_30)
        set_PIN1_SW_IN(choose_aspen, bit_32)
        set_PIN9_SW_IN(choose_aspen, bit_34)

    cpld_quit()
    print("\nCPLD Configure Finished.\n")


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        pass
    finally:
        exit(0)
